package com.aliyun.chatbot.channel.dingtalk;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import com.alibaba.fastjson2.JSON;

import com.aliyun.chatbot.channel.base.ChannelMsgSender;
import com.aliyun.dingtalkcard_1_0.Client;
import com.aliyun.dingtalkcard_1_0.models.CreateAndDeliverHeaders;
import com.aliyun.dingtalkcard_1_0.models.CreateAndDeliverRequest;
import com.aliyun.dingtalkcard_1_0.models.CreateAndDeliverResponse;
import com.aliyun.dingtalkcard_1_0.models.StreamingUpdateHeaders;
import com.aliyun.dingtalkcard_1_0.models.StreamingUpdateRequest;
import com.aliyun.dingtalkcard_1_0.models.StreamingUpdateResponse;
import com.aliyun.teaopenapi.models.Config;
import com.aliyun.teautil.models.RuntimeOptions;
import lombok.extern.slf4j.Slf4j;

/**
 * @author kelin
 * @date 2024/6/7
 */
@Slf4j
public class DingtalkChannelMsgSender extends ChannelMsgSender<DingtalkChatContext> {

    private final Client client;
    private final String templateId;

    public DingtalkChannelMsgSender(DingtalkChatContext context, String templateId) {
        super(context);
        this.templateId = templateId;
        this.client = createClient();
    }

    protected Client createClient() {
        try {
            Config config = new Config();
            config.protocol = "https";
            config.regionId = "central";
            config.endpoint = "api.dingtalk.com";
            return new Client(config);
        } catch (Exception e) {
            log.error("createClient get excpetion, msg:{}", e.getMessage());
        }
        return null;
    }

    @Override
    public void preSengMsg() {
        try {
            CreateAndDeliverHeaders headers
                = new CreateAndDeliverHeaders();
            headers.xAcsDingtalkAccessToken = context.getDingtalkAccessTokenClient().getAccessToken();

            CreateAndDeliverRequest.CreateAndDeliverRequestCardData cardData =
                new CreateAndDeliverRequest.CreateAndDeliverRequestCardData();

            CreateAndDeliverRequest request
                = new CreateAndDeliverRequest()
                .setOutTrackId(context.getOutTrackId())
                .setCardTemplateId(templateId)
                .setCardData(cardData)
                .setOpenSpaceId(getOpenSpaceId(context.getDingtalkInputMessage()))
                .setUserIdType(1);
            if (context.getDingtalkInputMessage().isGroup()) {
                groupCard(context.getRobotCode(), request);
            } else {
                singleCard(context.getRobotCode(), request);
            }

            CreateAndDeliverResponse resp = client.createAndDeliverWithOptions(request, headers,
                new RuntimeOptions());
            log.info(" DingtalkChannelMsgSender#preSengMsg createAndDeliverWithOptions request:{}, resp:{}",
                JSON.toJSONString(request),
                JSON.toJSONString(resp));
        } catch (Exception e) {
            log.warn(" DingtalkChannelMsgSender#preSengMsg get exception, msg:{}", e.getMessage());
        }
    }

    @Override
    public void sendMsg(String reply, Boolean streamEnd) throws Exception {
        try {
            StreamingUpdateHeaders headers = new StreamingUpdateHeaders();
            headers.xAcsDingtalkAccessToken = context.getDingtalkAccessTokenClient().getAccessToken();
            StreamingUpdateRequest request = new StreamingUpdateRequest()
                .setOutTrackId(context.getOutTrackId())
                .setGuid(UUID.randomUUID().toString())
                .setKey("content")
                .setContent(reply)
                .setIsFull(true).setIsFinalize(streamEnd);
            StreamingUpdateResponse response = client.streamingUpdateWithOptions(request, headers,
                new RuntimeOptions());
            log.info(" DingtalkChannelMsgSender#sendMsg get resp:{}", JSON.toJSONString(response));

        } catch (Exception e) {
            log.error(" DingtalkChannelMsgSender#sendMsg get exception, msg:{}", e.getMessage());
        }
    }

    @Override
    public void postSendMsg() {

    }

    protected String getOpenSpaceId(DingtalkInputMessage dingtalkInputMessage) {
        if (dingtalkInputMessage.isGroup()) {
            return "dtv1.card//IM_GROUP." + dingtalkInputMessage.getConversationId();
        }
        return "dtv1.card//IM_ROBOT." + dingtalkInputMessage.getSenderStaffId();
    }

    private void groupCard(String robotCode, CreateAndDeliverRequest request) {
        CreateAndDeliverRequest.CreateAndDeliverRequestImGroupOpenDeliverModel imGroupOpenDeliverModel =
            new CreateAndDeliverRequest.CreateAndDeliverRequestImGroupOpenDeliverModel()
                .setRobotCode(robotCode);

        Map<String, String> lastMsgI18n = new HashMap<>();
        lastMsgI18n.put("ZH_CN", "助理回复");
        lastMsgI18n.put("EN_US", "Assistant reply");

        CreateAndDeliverRequest.CreateAndDeliverRequestImGroupOpenSpaceModel imGroupOpenSpaceModel =
            new CreateAndDeliverRequest.CreateAndDeliverRequestImGroupOpenSpaceModel()
                .setLastMessageI18n(lastMsgI18n)
                .setSupportForward(true);
        request.setImGroupOpenSpaceModel(imGroupOpenSpaceModel)
            .setImGroupOpenDeliverModel(imGroupOpenDeliverModel);
    }

    private void singleCard(String robotCode, CreateAndDeliverRequest request) {
        CreateAndDeliverRequest.CreateAndDeliverRequestImRobotOpenDeliverModel imRobotOpenDeliverModel =
            new CreateAndDeliverRequest.CreateAndDeliverRequestImRobotOpenDeliverModel()
                .setRobotCode(robotCode);

        Map<String, String> lastMsgI18n = new HashMap<>();
        lastMsgI18n.put("ZH_CN", "助理回复");
        lastMsgI18n.put("EN_US", "Assistant reply");

        CreateAndDeliverRequest.CreateAndDeliverRequestImRobotOpenSpaceModel imRobotOpenSpaceModel =
            new CreateAndDeliverRequest.CreateAndDeliverRequestImRobotOpenSpaceModel()
                .setLastMessageI18n(lastMsgI18n)
                .setSupportForward(true);
        request.setImRobotOpenSpaceModel(imRobotOpenSpaceModel)
            .setImRobotOpenDeliverModel(imRobotOpenDeliverModel);
    }
}
